<?php
/**
 * OOPでBBS (Step7)
 *
 * オブジェクト指向プログラミングで掲示板を作ってみる。
 * ブログでその作り方を晒すために書いたものです。
 * 詳しい事は当該記事を見てください。 
 *
 *
 *           
 * PHP version 5.3
 *
 * LICENSE : このソースはCreative Commons 表示 - 非営利 2.1 日本 Licenseです。
 *           改変は自由ですが営利目的で利用することは出来ません。
 * 
 * @category  Step7
 * @package   OOPdeBBS
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 *
 */


/**
 * 掲示板のデータを操作するクラス
 *
 * ModelはControllerに使役されるクラスです。
 * 掲示板のデータに関する手続きを行います。
 *
 * <step7>
 * editとdeleteメソッドを追加した。
 * writeとeditに共通するソースをaddLogStatusメソッドに移した。
 * 
 * @author    Tenderfeel <tenderfeel@gmail.com>
 * @copyright Copyright (c) 2010 Tenderfeel.(http://webtecnote.com)
 * @license   Creative Commons BY-NC http://creativecommons.org/licenses/by-nc/2.1/jp/
 * @since     File available since Release 1.0.0 (step1)
 */
 

class Model {
    
	//const DATA_DIRECTORY = 'dat/'; @deprecated 1.5 (step5)
	private $_dataDirectory;    //データディレクトリ
    private $_logFilename;      //ログファイル名
	private $_logFileExtention; //ログファイルの拡張子
    private $_inputs;           //フォームのnameキー
	private $_validationRule;   //バリデーションルール
    private $_post;             //$_POSTが入る
    private $_error;            //エラーが入る
	private $_logs;             //ログが入る
	protected $_userControll;      //ユーザー操作許可
    
    public function __construct($config)
    {
		
		$this->_dataDirectory = $config->get('data_directory') . DIRECTORY_SEPARATOR;
		
		$this->_logFileExtention = '.' . $config->get('logfile_extention');
        $this->_logFilename = '';             
		
        $this->_inputs = $config->getInputs();
		$this->_validationRule = $config->get('validation');
		
		if($config->get('user_controll')==='1'){//ユーザーによる操作
			$this->_userControll = true;
			$this->_inputs[]='key';//キーを追加
			$this->_validationRule['key']='キー,notEmpty,alphaNumeric';
		}
		
        $this->_post = array();               
        $this->_error = NULL;
        $this->_logs = NULL;
    }
    
    /**
     * 仮のメッセージを表示する
     * @return {string} HelloWorldメッセージを返す。
     * @since 1.0 (step1)
     * @deprecated 1.3 (step3)
     */
    //public function dispatch()
    //{
    //    return 'Hello Model!';
    //}
    
    /**
     * 送信されたデータのセッター
     * @param {array} POST配列
     * @return {}
     * @since 1.3 (step3)
     */
    private function _setPost($data)
    {
        $this->_post = $data;
    }
    
    /**
     * 送信されたデータのゲッター
     * @return {array}
     * @since 1.3 (step3)
     */
    private function _getPost()
    {
        return $this->_post;
    }
    
    /**
     * エラーのセッター
     * @return {array}
     * @since 1.3 (step3)
     */
    public function setError($error)
    {
        $this->_error = $error;
    }
    /**
     * エラーのゲッター
     * @return {array}
     * @since 1.3 (step3)
     */
    public function getError()
    {
        return $this->_error;
    }
    
    /**
     * ログファイル名のセッター
     * @param {string} $name ファイル名（拡張子なし）
     * @return {}
     * @since 1.3 (step3)
     */
    public function setLogfileName($name)
    {
        $this->_logFilename = $name . $this->_logFileExtention;
    }
    /**
     * ログファイル名のゲッター
     * @return {array}
     * @since 1.3 (step3)
     */
    private function _getLogfileName()
    {
        return $this->_dataDirectory . $this->_logFilename;
    }
    
    /**
     * ログデータのセッター
     * @param {array} ログ配列
     * @return {}
     * @since 1.4
     */
    public function setLogData($log)
    {
        $this->_logs = $log;
    }
    
    /**
     * ログデータのゲッター
     * @return {array} ログ配列
     * @since 1.4
     */
    public function getLogData()
    {
        return $this->_logs;
    }
    
    /**
     * 送信された内容をメンバ変数にセット
     * @param {array} $posts $_POST配列
     * @since 1.3 (step3)
     */
    public function setPostdata($posts)
    {
        foreach($this->_inputs as $target){
            $data[$target] = $posts[$target];
        }
        
        $this->_setPost($data);//セッターに渡す
        
        return;
    }
    
    /**
     * データのバリデーション
     * 
     * 入力されているかどうか確認。
     * 
     * @param {array} POST配列
     * @return {array} エラー配列 
     * @since 1.3 (step3)
     */
    public function validation()
    {
        $data = $this->_getPost();
        $error = array();
        
        foreach($data as $key => $value){
			
			if(!empty($this->_validationRule[$key])){
				
				//$this->_validationRule key = label, Rule1, Rule2...
				list($label, $rules) = explode(',', $this->_validationRule[$key], 2);
				$rules = explode(',', $rules);
				
				foreach($rules as $rule){
					$rule = trim($rule);
					if($rule == 'notEmpty' && $value == ''){
						
						$error[] = $label.'は必ず入力してください';
						 
					}elseif($rule == 'alphaNumeric' 
							&& $value != '' 
							&& !preg_match('/^[\d\w]+$/i', $value)){
						
						$error[] = $label.'は半角英数字のみ使用できます。'; 
						
					}elseif($rule == 'email' 
							&& $value != '' 
							&& !preg_match("/^([a-z0-9_]|\-|\.|\+)+@(([a-z0-9_]|\-)+\.)+[a-z]{2,6}$/i",$value) ){
						
						$error[] = $label.'の書式に誤りがあります。'; 
						
					}elseif($rule == 'url' 
							&& $value != '' 
							&& !preg_match('/^(https|http)(:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+)$/', $value)){
						$error[] = $label.'の書式に誤りがあります。'; 
					}
				}
			}
        }
        
        $this->setError($error);
        
        if(count($error)>0){
            return false;
        }else{
            return true;
        }
    }
	
	/**
	 * ログＩＤの算出
	 * @param {array} $log ログの配列（file関数の戻り値）
	 */
	private function _calcPostID($log)
	{
		$id = 0;
		
		if(!empty($log)){
			list($id, $str) = explode(',', $log[0], 2);
		}
		
		return $id + 1;
		
	}
    
    /**
     * データの保存
     * 
     * csv形式でログファイルに書き込む。
     * 
     * @return {boolean} 成功時にtrue
     * @since 1.3 (step3)
     */
    public function write()
    {
         
		$log = file($this->_getLogfileName());//ログ確保
        $log = implode('', $log);
		
		$id = $this->_calcPostID($log);
		
        $data = $this->addLogStatus($id);//IDとタイムスタンプ追加、key暗号化
        
		//$log = file_get_contents($this->_getLogfileName());//deprecated 1.6 (step6)

        $fp = fopen($this->_getLogfileName(), 'w');
    
        $this->myFputcsv($fp, $data);
        fwrite($fp, trim($log));
        
        fclose($fp);
        
        return true;
    }
    
    /**
     * 新しいログにIDやdate情報を加える
     * userControll=trueの場合はkeyの暗号化もする
     * @param {number|string} $id ログID
     * @since 1.7 (step7)
     */
    protected function addLogStatus($id){
        
        $data = $this->_getPost();
        
        $data = array_map(array($this,'clean'), $data);//クリーンアップ
        
        array_unshift($data, $id, time());//先頭にIDとタイムスタンプ
        
        if($this->_userControll){//キー暗号化
            $data['key'] = sha1($data['key']);
        }
        
        return $data;
    }
    
    /**
     * 値をクリーンアップする
     * @param {string} $str フォームで入力された文字列
     * @param {string} タグとか除去した文字列
     * @since 1.3 (step3)
     */
    public function clean($str)
    {
        $str = strip_tags(trim($str));
        $str = preg_replace('/<(script|style).*?<\/(script|style)>/sm', '', $str);//script
        $str = preg_replace("/<\!--.*-->/sm", '', $str); //comments
        $str = htmlentities($str,ENT_QUOTES,"UTF-8");
        $str = str_replace(array("\r\n", "\r", "\n\n", "\n", "\0"), "", nl2br($str));
        $str = str_replace(array("  ", ",", '"'), array("\t", '&#044;', '&quot;'), $str);
        
        if (get_magic_quotes_gpc()) {
            $str = stripslashes($str);
        }
        
        return $str;
        
    }
    
    /**
     * エンクロージャー問題解決用のfputcsv
     *
     * 日本語文字列が囲まれてないとfgetcsvしたとき文字化けする。
     * オリジナルのfputcsvはエスケープ文字が含まれているときしか囲ってくれないから
     * 半角英数以外の文字が含まれていたら$enclosureで囲むようにする。
     * マニュアルのコメントから拝借したソースをカスタマイズ。
     * 
     * @param {fp} $handle ファイルポインタ
     * @param {array} $fields 値の配列
     * @param {string} $delimiter フィールド区切り文字 
     * @param {string} $enclosure フィールドを囲む文字
     * @return {numeric|false} 書き込んだバイト数、またはエラー時に FALSE（fwriteと同じ）
     * @since 1.3 (step3)
     */
    private function myFputcsv($handle, $fields, $delimiter=',', $enclosure='"', $mysql_null = false)
    {
        $delimiter_esc = preg_quote($delimiter, '/');
        $enclosure_esc = preg_quote($enclosure, '/');
    
        $output = array();
        foreach ($fields as $field) {
            if ($field === null && $mysql_null) {
                $output[] = 'NULL';
                continue;
            }
    
            $output[] = preg_match("/(?:${delimiter_esc}|${enclosure_esc}|\s|[^\x01-\x7e])/", $field) 
                        ? ($enclosure . str_replace($enclosure, $enclosure . $enclosure, $field) . $enclosure)
                        : $field;
        }
    
        return fwrite($handle, join($delimiter, $output) . "\n"); 
    }

    /**
     * ログファイルからログデータを読み込む
     * 
     * fgetcsvの戻り値をフォームのnameと一緒にLogクラスへ渡してオブジェクト化する。
     * 
     * @return {boolean}
     * @since 1.4(Step4)
     */
    public function readLog()
    {
        $logs = array();
        $label = $this->_inputs;
        array_unshift($label, 'id', 'date');
        
        if (($handle = fopen($this->_getLogfileName(), "r")) !== FALSE) {
            while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
                $logs[] = new Log($label, $data);
            }
            fclose($handle);
        }else{
            return false;
        }
        
        $this->setLogData($logs);
        
        return true;
    }
	
	/**
     * ＩＤで指定されたログを探す
     * 
     * @return {object|false} Logクラスインスタンス
     * @since 1.6(Step6)
     */
	public function findData($id)
	{
		$logs = $this->getLogData();
		
		if(empty($id)) return false;
		
		foreach($logs as $log){
			if($log->id == $id) return $log;
		}
		
	}
    
    /**
     * ログの削除
     * @param {number|string} $id 記事ID
     * @return
     * @since 1.7 (step7)
     */
    public function delete($id)
    {
        if(empty($id) || !$this->getLogData()) return false;
        
        $new = array();//新しいログ
        $logs = file($this->_getLogfileName());//ログ確保
        
        $fp = fopen($this->_getLogfileName(), 'w');
        
        foreach($logs as $i => $log){
            list($pid, $str) = split(',', $log, 2);
            if($pid != $id){
                fwrite($fp, trim($new));
            }
        }
        
        fclose($fp);
        
        return true;
    }
    
    /**
     *　ログの編集
     * @param {number|string} $id 記事ID
     * @return
     * @since 1.7 (step7)
     */
    public function edit($id)
    {
        if(empty($id) || !$this->getLogData()) return false;
        
        $data = $this->addLogStatus($id);//IDとタイムスタンプ追加、key暗号化
        
        $logs = file($this->_getLogfileName());//ログ確保

        $fp = fopen($this->_getLogfileName(), 'w');
        
        foreach($logs as $i => $log){
            list($pid, $str) = split(',', $log, 2);
            if($pid == $id){
                $this->myFputcsv($fp, $data);
            }else{
                fwrite($fp, trim($log));
            }
        }
        
        fclose($fp);
        
        return true;
    }
}

?>